@@ -32,10 +32,12 @@
 int __connman_dbus_init(DBusConnection *conn);
 void __connman_dbus_cleanup(void);
 
-DBusMessage *__connman_error_failed(DBusMessage *msg);
+DBusMessage *__connman_error_failed(DBusMessage *msg, int errnum);
 DBusMessage *__connman_error_invalid_arguments(DBusMessage *msg);
 DBusMessage *__connman_error_permission_denied(DBusMessage *msg);
 DBusMessage *__connman_error_not_supported(DBusMessage *msg);
+DBusMessage *__connman_error_not_implemented(DBusMessage *msg);
+DBusMessage *__connman_error_no_carrier(DBusMessage *msg);
 
 int __connman_selftest(void);
 
@@ -52,8 +54,10 @@ int __connman_profile_init(DBusConnection *conn);
 void __connman_profile_cleanup(void);
 
 void __connman_profile_list(DBusMessageIter *iter);
-void __connman_profile_list_services(DBusMessageIter *iter);
-const char *__connman_profile_active(void);
+const char *__connman_profile_active_ident(void);
+const char *__connman_profile_active_path(void);
+
+void __connman_profile_changed(void);
 
 #include <connman/log.h>
 
@@ -78,6 +82,8 @@ int __connman_security_check_privilege(DBusMessage *message,
 const char *__connman_ipv4_method2string(enum connman_ipv4_method method);
 enum connman_ipv4_method __connman_ipv4_string2method(const char *method);
 
+#include <connman/ipconfig.h>
+
 #include <connman/resolver.h>
 
 int __connman_resolver_init(void);
@@ -93,9 +99,12 @@ void __connman_storage_cleanup(void);
 int __connman_storage_init_device();
 int __connman_storage_load_device(struct connman_device *device);
 int __connman_storage_save_device(struct connman_device *device);
-int __connman_storage_init_network();
+int __connman_storage_init_network(struct connman_device *device);
 int __connman_storage_load_network(struct connman_network *network);
 int __connman_storage_save_network(struct connman_network *network);
+int __connman_storage_init_service();
+int __connman_storage_load_service(struct connman_service *service);
+int __connman_storage_save_service(struct connman_service *service);
 
 #include <connman/driver.h>
 
@@ -123,8 +132,10 @@ void __connman_element_list(struct connman_element *element,
 int __connman_element_count(struct connman_element *element,
 					enum connman_element_type type);
 
-const char *__connman_element_get_device(struct connman_element *element);
-const char *__connman_element_get_network(struct connman_element *element);
+struct connman_service *__connman_element_get_service(struct connman_element *element);
+struct connman_device *__connman_element_get_device(struct connman_element *element);
+const char *__connman_element_get_device_path(struct connman_element *element);
+const char *__connman_element_get_network_path(struct connman_element *element);
 
 const char *__connman_element_type2string(enum connman_element_type type);
 
@@ -175,10 +186,14 @@ void __connman_device_decrease_connections(struct connman_device *device);
 void __connman_device_set_network(struct connman_device *device,
 					struct connman_network *network);
 
-void __connman_device_disconnect(struct connman_device *device);
+int __connman_device_connect(struct connman_device *device);
+int __connman_device_disconnect(struct connman_device *device);
 
 connman_bool_t __connman_device_has_driver(struct connman_device *device);
 
+const char *__connman_device_get_type(struct connman_device *device);
+const char *__connman_device_get_ident(struct connman_device *device);
+
 int __connman_device_set_offlinemode(connman_bool_t offlinemode);
 
 int __connman_profile_add_device(struct connman_device *device);
@@ -196,9 +211,31 @@ int __connman_network_disconnect(struct connman_network *network);
 
 connman_bool_t __connman_network_has_driver(struct connman_network *network);
 
+const char *__connman_network_get_type(struct connman_network *network);
+const char *__connman_network_get_group(struct connman_network *network);
+const char *__connman_network_get_ident(struct connman_network *network);
+
 int __connman_profile_add_network(struct connman_network *network);
 int __connman_profile_remove_network(struct connman_network *network);
 
+#include <connman/service.h>
+
+int __connman_service_init(void);
+void __connman_service_cleanup(void);
+
+void __connman_service_list(DBusMessageIter *iter);
+
+struct connman_service *__connman_service_lookup_from_device(struct connman_device *device);
+struct connman_service *__connman_service_create_from_device(struct connman_device *device);
+
+struct connman_service *__connman_service_lookup_from_network(struct connman_network *network);
+struct connman_service *__connman_service_create_from_network(struct connman_network *network);
+
+int __connman_service_set_carrier(struct connman_service *service,
+						connman_bool_t carrier);
+int __connman_service_indicate_state(struct connman_service *service,
+					enum connman_service_state state);
+
 #include <connman/notifier.h>
 
 int __connman_notifier_init(void);
