@@ -272,11 +272,61 @@ int __connman_element_count(struct connman_element *element,
 	return data.count;
 }
 
-const char *__connman_element_get_device(struct connman_element *element)
+static struct connman_network *__connman_element_get_network(struct connman_element *element)
+{
+	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
+						element->network != NULL)
+		return element->network;
+
+	if (element->parent == NULL)
+		return NULL;
+
+	return __connman_element_get_network(element->parent);
+}
+
+struct connman_service *__connman_element_get_service(struct connman_element *element)
+{
+	struct connman_service *service;
+	struct connman_network *network;
+	struct connman_device *device;
+	enum connman_device_type type;
+
+	device = __connman_element_get_device(element);
+	if (device == NULL)
+		return NULL;
+
+	type = connman_device_get_type(device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return NULL;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+		service = __connman_service_lookup_from_device(device);
+		break;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		network = __connman_element_get_network(element);
+		if (network == NULL)
+			return NULL;
+		service = __connman_service_lookup_from_network(network);
+		break;
+	}
+
+	return service;
+}
+
+struct connman_device *__connman_element_get_device(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
 						element->device != NULL)
-		return element->path;
+		return element->device;
 
 	if (element->parent == NULL)
 		return NULL;
@@ -284,7 +334,18 @@ const char *__connman_element_get_device(struct connman_element *element)
 	return __connman_element_get_device(element->parent);
 }
 
-const char *__connman_element_get_network(struct connman_element *element)
+const char *__connman_element_get_device_path(struct connman_element *element)
+{
+	struct connman_device *device;
+
+	device = __connman_element_get_device(element);
+	if (device == NULL)
+		return NULL;
+
+	return connman_device_get_path(device);
+}
+
+const char *__connman_element_get_network_path(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
 						element->network != NULL)
@@ -293,7 +354,7 @@ const char *__connman_element_get_network(struct connman_element *element)
 	if (element->parent == NULL)
 		return NULL;
 
-	return __connman_element_get_network(element->parent);
+	return __connman_element_get_network_path(element->parent);
 }
 
 static gint compare_priority(gconstpointer a, gconstpointer b)
@@ -433,6 +494,8 @@ void __connman_element_initialize(struct connman_element *element)
 
 	element->name    = NULL;
 	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
+	element->state   = CONNMAN_ELEMENT_STATE_UNKNOWN;
+	element->error   = CONNMAN_ELEMENT_ERROR_UNKNOWN;
 	element->index   = -1;
 	element->enabled = FALSE;
 
@@ -849,7 +912,7 @@ int connman_element_set_blob(struct connman_element *element,
 			const char *key, const void *data, unsigned int size)
 {
 	return set_static_array_property(element, key,
-						DBUS_TYPE_BYTE, data, size);
+						DBUS_TYPE_BYTE, &data, size);
 }
 
 /**
@@ -1239,6 +1302,28 @@ int connman_element_set_enabled(struct connman_element *element,
 	return 0;
 }
 
+/**
+ * connman_element_set_error:
+ * @element: element structure
+ * @error: error identifier
+ *
+ * Set error state and specific error identifier
+ */
+void connman_element_set_error(struct connman_element *element,
+					enum connman_element_error error)
+{
+	DBG("element %p error %d", element, error);
+
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return;
+
+	element->state = CONNMAN_ELEMENT_STATE_ERROR;
+	element->error = error;
+
+	if (element->driver && element->driver->change)
+		element->driver->change(element);
+}
+
 int __connman_element_init(DBusConnection *conn, const char *device,
 							const char *nodevice)
 {
@@ -1260,6 +1345,7 @@ int __connman_element_init(DBusConnection *conn, const char *device,
 	element_root = g_node_new(element);
 
 	__connman_notifier_init();
+	__connman_service_init();
 	__connman_network_init();
 	__connman_device_init();
 
@@ -1344,6 +1430,7 @@ void __connman_element_cleanup(void)
 
 	__connman_device_cleanup();
 	__connman_network_cleanup();
+	__connman_service_cleanup();
 	__connman_notifier_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
